package lcsb.mapviewer.web.api.project.mirnas;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.annotation.data.MiRNA;
import lcsb.mapviewer.annotation.services.TaxonomyBackend;
import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.modelutils.serializer.model.map.ElementIdentifierType;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.IElementService;
import lcsb.mapviewer.services.interfaces.IProjectService;
import lcsb.mapviewer.services.search.DbSearchCriteria;
import lcsb.mapviewer.services.search.mirna.IMiRNAService;
import lcsb.mapviewer.web.api.NewApiResponseSerializer;

@RestController
@RequestMapping(
    value = {
        "/minerva/new_api/projects/{projectId}/"
    },
    produces = MediaType.APPLICATION_JSON_VALUE)
public class NewMiRnaController extends BaseController {

  private IProjectService projectService;
  private IMiRNAService miRnaService;
  private IElementService elementService;
  private NewApiResponseSerializer serializer;

  @Autowired
  public NewMiRnaController(final NewApiResponseSerializer serializer, final IProjectService projectService, final IMiRNAService miRnaService,
      final IElementService elementService) {
    this.projectService = projectService;
    this.miRnaService = miRnaService;
    this.elementService = elementService;
    this.serializer = serializer;
  }

  @PreAuthorize("hasAnyAuthority('IS_ADMIN', 'READ_PROJECT:' + #projectId)")
  @GetMapping(value = "miRnas:search")
  public ResponseEntity<?> getMiRnasByQuery(
      final @PathVariable(value = "projectId") String projectId,
      final @RequestParam(value = "query", defaultValue = "") String query,
      final @RequestParam(value = "target", defaultValue = "") String target) throws QueryException, IOException, ObjectNotFoundException {
    List<MiRNA> data = new ArrayList<>();
    Project project = projectService.getProjectByProjectId(projectId);
    if (project == null) {
      throw new ObjectNotFoundException("Project with given id doesn't exist");
    }
    MiriamData organism = project.getOrganism();
    if (organism == null) {
      organism = TaxonomyBackend.HUMAN_TAXONOMY;
    }

    if (!query.equals("")) {
      MiRNA miRna = miRnaService.getByName(query, new DbSearchCriteria().project(project).organisms(organism).colorSet(0));
      if (miRna != null) {
        data.add(miRna);
      }
    } else if (target.contains(":")) {
      String targetType = target.split(":", -1)[0];
      String targetId = target.split(":", -1)[1];

      Integer elementId = Integer.valueOf(targetId);
      List<Element> targets = new ArrayList<>();
      if (targetType.equals(ElementIdentifierType.ALIAS.getJsName())) {
        targets.add(elementService.getElementById(project.getProjectId(), elementId));
      } else {
        throw new QueryException("Targeting for the type not implemented");
      }

      data = miRnaService.getForTargets(targets, new DbSearchCriteria().project(project).organisms(organism));
    } else {
      data = new ArrayList<>();
    }
    return serializer.prepareResponse(data);
  }
}